/**********************************************************************
 * options.c                                            Sepmtember 1999
 * Horms                                             horms@vergenet.net
 *
 * Read in command line options
 * Code based on man getopt(3)
 *
 * Released under the terms of the GNU GPL
 **********************************************************************/

#include "options.h"

int idle_timeout;


/**********************************************************************
 * options
 * Read in command line options
 **********************************************************************/

options_t options(int argc, char **argv){
  int c;
  options_t options;
  char * short_options_string="acd:s:Sm:il:p:P:u:g:hjt:";

  extern int idle_timeout;

  idle_timeout                       =DEFAULT_IDLE_TIMEOUT;
  options.authenticate_in            =DEFAULT_AUTHENTICATE_IN;
  options.domain_delimiter           =DEFAULT_DOMAIN_DELIMITER;
  options.client_server_specification=DEFAULT_CLIENT_SERVER_SPECIFICATION;
  options.group                      =DEFAULT_GROUP;
  options.inetd_mode                 =DEFAULT_INETD_MODE;
  options.listen_port                =PERDITION_PORT_PROTOCOL_DEPENDANT;
  options.mapname                    =DEFAULT_MAPNAME;
  options.outgoing_server            =DEFAULT_OUTGOING_SERVER;
  options.outgoing_port              =PERDITION_PORT_PROTOCOL_DEPENDANT;
  options.protocol                   =DEFAULT_PROTOCOL;
  options.strip_domain               =DEFAULT_STRIP_DOMAIN;
  options.username                   =DEFAULT_USERNAME;

  while (1){
#ifdef HAVE_GETOPT_LONG
    int option_index = 0;
    static struct option long_options[] =
    {
      {"authenticate_in", 0, 0, 0},
      {"client_server_specification", 0, 0, 0},
      {"domain_delimiter", 1, 0, 0},
      {"group", 1, 0, 0},
      {"help", 0, 0, 0},
      {"inetd_mode", 0, 0, 0},
      {"jain", 2, 0, 0},
      {"jane", 2, 0, 0},
      {"jayne", 2, 0, 0},
      {"map_name", 1, 0, 0},
      {"outgoing_server", 1, 0, 0},
      {"outgoing_port", 1, 0, 0},
      {"protocol", 1, 0, 0},
      {"listen_port", 1, 0, 0},
      {"strip_domain", 0, 0, 0},
      {"timeout", 1, 0, 0},
      {"username", 1, 0, 0},
      {0, 0, 0, 0}
    };

    c = getopt_long (
      argc, 
      argv, 
      short_options_string, 
      long_options, 
      &option_index
    );
#else
    c = getopt (argc, argv, short_options_string);
#endif
    if (c == -1){
      goto leave;
    }

    switch (c){
#ifdef HAVE_GETOPT_LONG
      case 0:
        if(strcmp(long_options[option_index].name,"help")==0){
          usage(0);
	  break;
        }
        if(str_is_jain(long_options[option_index].name)){
          printf(
            "%s, Oath\n", 
            long_options[option_index].name
          );
          break;
        }
        if(strcmp(long_options[option_index].name,"map_name")==0){
          options.mapname=optarg;
          break;
        }
        if(strcmp(long_options[option_index].name,"inetd_mode")==0){
          options.inetd_mode=1;
          break;
        }
        if(strcmp(
	  long_options[option_index].name,
	  "domain_delimiter"
	)==0){
          options.domain_delimiter=optarg;
          break;
        }
        if(strcmp(
	  long_options[option_index].name,
	  "outgoing_server"
	)==0){
          options.outgoing_server=optarg;
          break;
        }
        if(strcmp(
	  long_options[option_index].name,
	  "outgoing_port"
	)==0){
          options.outgoing_port=optarg;
          break;
        }
        if(strcmp(
	  long_options[option_index].name,
	  "client_server_specification"
	)==0){
          options.client_server_specification=1;
          break;
        }
        if(strcmp(
	  long_options[option_index].name,
	  "strip_domain"
	)==0){
          options.strip_domain=1;
          break;
        }
        if(strcmp(
	  long_options[option_index].name,
	  "authenticate_in"
	)==0){
          options.authenticate_in=1;
          break;
        }
        if(strcmp(long_options[option_index].name,"listen_port")==0){
          options.listen_port=optarg;
          break;
        }
        if(strcmp(long_options[option_index].name,"protocol")==0){
          if((options.protocol=protocol_index(optarg))<0){
            syslog(LOG_WARNING, "options: invalid protocol: %s", optarg);
            usage(-1);
          }
          break;
        }
        if(strcmp(long_options[option_index].name,"username")==0){
          options.username=optarg;
          break;
        }
        if(strcmp(long_options[option_index].name,"group")==0){
          options.group=optarg;
          break;
        }
        if(strcmp(long_options[option_index].name,"timeout")==0){
          if(!str_is_digit(optarg)){
            usage(-1);
          }
          idle_timeout=atoi(optarg);
          break;
        }
        break;
#endif
      case 'h':
        usage(0);
      case 'j':
        printf("Jain, Oath\n");
        break;
      case 'm':
        options.mapname=optarg;
        break;
      case 'i':
        options.inetd_mode=1;
        break;
      case 'c':
        options.client_server_specification=1;
        break;
      case 'd':
        options.domain_delimiter=optarg;
        break;
      case 'S':
        options.strip_domain=1;
        break;
      case 's':
        options.outgoing_server=optarg;
        break;
      case 'a':
        options.authenticate_in=1;
        break;
      case 'p':
        options.outgoing_port=optarg;
        break;
      case 'l':
        options.listen_port=optarg;
        break;
      case 'P':
        if((options.protocol=protocol_index(optarg))<0){
          syslog(LOG_WARNING, "options: invalid protocol: %s", optarg);
          usage(-1);
        }
        break;
      case 'u':
        options.username=optarg;
        break;
      case 'g':
        options.group=optarg;
        break;
      case 't':
        if(!str_is_digit(optarg)){
          usage(-1);
        }
        idle_timeout=atoi(optarg);
        break;
      case '?':
        usage(-1);
      default:
        printf ("?? getopt returned character code 0%o ??\n", c);
      }
  }

  if (optind<argc) {
    usage(-1);
  }

  leave:
  options.domain_delimiter_length=
    strlen(options.domain_delimiter);

  return(options);
}


/**********************************************************************
 * log_options
 * Log options to syslog
 **********************************************************************/

int log_options(options_t options){
  char *protocol=NULL;

  extern int idle_timeout;
  extern struct utsname *system_uname;

  if((protocol=protocol_list(protocol, NULL, options.protocol))==NULL){
    syslog(LOG_WARNING, "log_options: protocol_list");
    return(-1);
  }

  syslog(
    LOG_INFO, 
    "authenticate_in=%d, "
    "client_port_specification=%d, "
    "domain_delimiter=\"%s\", "
    "group=\"%s\", "
    "inetd_mode=%d, "
    "listen_port=\"%s\", "
    "map_name=\"%s\", "
    "nodename=\"%s\", "
    "outgoing_port=\"%s\", "
    "outgoing_server=\"%s\", "
    "prototol=\"%s\", "
    "strip_domain=%d, "
    "timeout=%d, "
    "username=\"%s\"\n",
    options.authenticate_in,
    options.client_server_specification,
    str_null_safe(options.domain_delimiter),
    str_null_safe(options.group),
    options.inetd_mode,
    str_null_safe(options.listen_port),
    str_null_safe(options.mapname),
    str_null_safe(system_uname->nodename),
    str_null_safe(options.outgoing_port),
    str_null_safe(options.outgoing_server),
    str_null_safe(protocol),
    options.strip_domain,
    idle_timeout,
    str_null_safe(options.username)
  );

  free(protocol);
  return(0);
}


/**********************************************************************
 * usage
 * Display usage unfromtaion
 * Printed to stdout if exit_status=0, stderr otherwise
 **********************************************************************/

void usage(int exit_status){
  FILE *stream;
  char *available_protocols=NULL;
  char *default_protocol_str=NULL;

  if(exit_status!=0){
     stream=stderr;
  }
  else{
     stream=stdout;
  }
  
  if((available_protocols=protocol_list(
    available_protocols, 
    ", ", 
    PROTOCOL_ALL
  ))==NULL){
    syslog(LOG_WARNING, "usage: protocol_list 1");
    available_protocols="*error*";
  }

  if((default_protocol_str=protocol_list(
    default_protocol_str, 
    NULL,
    PROTOCOL_DEFAULT
  ))==NULL){
    syslog(LOG_WARNING, "usage: protocol_list 2");
    default_protocol_str="*error*";
  }

  fprintf(
    stream, 
    "Usage: perdition [options]\n"
    "  options:\n"
#ifdef HAVE_GETOPT_LONG    
    "     -a|--authenticate_in: \n"
    "                      User is authenticated by perdition before\n"
    "                      connection to backend server is made\n"
    "     -c|--client_server_specification: \n"
    "                      Allow USER of the form\n"
    "                      user<delimiter>server[:port] to\n"
    "                      specify the server and port for a user\n"
    "                      Note: over-rides -s|--strip_domain\n"
    "     -d|--domain_delimiter:\n"
    "                      Delimiter used for\n"
    "                      -c|--client_server_specification and\n"
    "                      -s|--strip_domain options. Multicharacter\n"
    "                      delimiters are permitted (default \"%s\")\n"
    "     -g|--group:      Group to run as (default \"%s\")\n"
    "     -h|--help:       Display this message\n"
    "     -i|--inetd_mode: Run in inetd mode\n"
    "     -l|--listen_port:\n"
    "                      Port to listen on. (default \"%s\")\n"
    "     -m|--map_name    gdbm map for users to hosts and port. If map is\n"
    "                      empty (\"\") then no lookup takes place.\n"
    "                      (default \"%s\")\n"
    "     -P|--protocol:   Protocol to use (default \"%s\")\n"
    "                      available protocols: \"%s\"\n"
    "     -p|--outgoing_port:\n"
    "                      Define a port to use if a port is not defined for\n"
    "                      a user in popmap. (default \"%s\")\n"
    "     -s|--outgoing_server:\n"
    "                      Define a server to use if a user is not in the\n"
    "                      popmap. (default \"%s\")\n"
    "     -S|--strip_domain:\n"
    "                      Allow USER of the from user<delimiter>domain where\n"
    "                      <delimiter>domain will be striped off\n"
    "                      Note: over-ridden by "
                           "-c|--client_server_specification\n"
    "     -t|--timeout:    Idle timeout in seconds. Value of zero sets\n"
    "                      infinite timeout. (default %d)\n"
    "     -u|--username:   Username to run as (default \"%s\")\n",
#else
    "     -a  User is authenticated by perdition before\n"
    "         connection to backend server is made\n"
    "     -c: Allow USER of the form user<delimiter>server[:port] to\n"
    "         specify the server and port for a user\n"
    "         Note: over-rides -s\n"
    "     -d  Delimiter used for -c and -s. \n"
    "         Multicharacter delimiters are permitted (default \"%s\")\n"
    "     -g: Group to run as (default \"%s\")\n"
    "     -h: Display this message\n"
    "     -i: Run in inetd mode\n"
    "     -l: Port to listen on. (default \"%s\")\n"
    "     -m: gdbm map for users to hosts and port. If map is empty then no\n"
    "         lookup takes place. (default \"%s\")\n"
    "     -P: Protocol to use (default \"%s\")\n"
    "         available protocols: \"%s\"\n"
    "     -p  Define a port to use if a port is not defined for a user\n"
    "         in popmap. (default \"%s\")\n"
    "     -s  Define a server to use if a user is not in the popmap.\n"
    "         (default \"%s\")\n"
    "     -S: Allow USER of the from user<delimiter>domain.com where\n"
    "         <delimiter>domain will be striped off\n"
    "         Note: over-ridden by -c\n"
    "     -t: Idle timeout in seconds. Value of zero gives infinite timeout.\n"
    "         (default %d)\n"
    "     -u: Username to run as (default \"%s\")\n",
#endif
    str_null_safe(DEFAULT_DOMAIN_DELIMITER),
    str_null_safe(DEFAULT_GROUP),
    str_null_safe(PERDITION_PORT_PROTOCOL_DEPENDANT),
    str_null_safe(DEFAULT_MAPNAME),
    str_null_safe(default_protocol_str),
    str_null_safe(available_protocols),
    str_null_safe(PERDITION_PORT_PROTOCOL_DEPENDANT),
    str_null_safe(DEFAULT_OUTGOING_SERVER),
    DEFAULT_IDLE_TIMEOUT,
    str_null_safe(DEFAULT_USERNAME)
  );

  daemon_exit_cleanly(exit_status);
}
