/**********************************************************************
 * tcp_socket_server.c                                  September 1999
 * Horms                                             horms@vergenet.net
 *
 * Open accpet a tcp connection from a client (we are the server)
 *
 * Licenced under the terms of the GNU GPL
 *
 **********************************************************************/

#include "tcp_socket_server.h"


/**********************************************************************
 * tcp_socker_server_connect
 * Listen on a tcp port for incoming client connections 
 * When one is recieved fork
 * In the Child: close the listening file desctiptor
 *               return the file descriptor that is 
 *               a socket connection to the client
 * In the Server: close the socket to the client and loop
 * Pre: port to listen to, an ascii representation of a number
 * Post: Client sockets are returned in child processes
 *       The function doesn't exit, other than on error in
 *       the parent process.
 **********************************************************************/

int tcp_socket_server_connect(char *port){
  int s;
  int g;
  int addrlen;
  struct sockaddr_in from;

  if((s=socket(AF_INET, SOCK_STREAM, 0)) < 0){
    syslog(
      LOG_WARNING, 
      "tcp_socket_server_connect: socket: %s, Exiting",
      strerror(errno)
    );
    daemon_exit_cleanly(1);
  }

  /* 
   * Set SO_REUSEADDR on the server socket s. Variable g is used
   * as a scratch varable.
   */
  g = 1;
  if(setsockopt(s,SOL_SOCKET,SO_REUSEADDR,(void *)&g,sizeof g) <0){
    syslog(
      LOG_WARNING,      
      "tcp_socket_server_connect: setsockopt: %s, Exiting",
      strerror(errno)
    );
    daemon_exit_cleanly(1);
   }

/* Set SO_REUSEADDR on the server socket s. Variable g is used
as a scratch varable.
*/
  g = 1;
  if (setsockopt(s,SOL_SOCKET,SO_REUSEADDR,(void *)&g,sizeof g) <0){
    syslog(
      LOG_WARNING,      "tcp_socket_server_connect: setsockopt: %s, Exiting",
      strerror(errno)
    );
    daemon_exit_cleanly(1);
   }

  from.sin_family = AF_INET;
  from.sin_port = htons(atoi(port));
  from.sin_addr.s_addr = htonl(INADDR_ANY);
  addrlen = sizeof(from);

  if(bind(s, (struct sockaddr *)&from, addrlen)<0){
    syslog(
      LOG_WARNING, 
      "tcp_socket_server_connect: bind: %s, Exiting",
      strerror(errno)
    );
    daemon_exit_cleanly(1);
  }

  signal(SIGCHLD, (void(*)(int))tcp_socket_server_reaper);
  listen(s, 5);

  for(;;){
    if((g=accept(s, (struct sockaddr *)&from, &addrlen))<0){
      syslog(
        LOG_WARNING, 
        "tcp_socket_server_connect: accept: %s",
        strerror(errno)
      );
    }
    else{
      if(fork()==0){
        if(close(s)<0){
          syslog(
	    LOG_WARNING, 
	    "tcp_socket_server_connect: close, Exiting",
	    strerror(errno)
	  );
          daemon_exit_cleanly(1);
        }
        return(g);
      }
      else{
        if(close(g)){
          syslog(
	    LOG_WARNING, 
	    "tcp_socket_server_connect: close: %s, Exiting",
	    strerror(errno)
	  );
          daemon_exit_cleanly(1);
        }
      }
    }
  }
}


/**********************************************************************
 * tcp_socker_server_reaper
 * A signal handler that waits for SIGCHLD and runs wait3 to free
 * the resources of any exited children. This stops zombie processes
 * from hanging around.
 * pre: SIGCHLD is recieved by the process
 * post: Resoerces of any exited children are freed
 * return: 0
 **********************************************************************/

int tcp_socket_server_reaper(void){
  union wait status;

  signal(SIGCHLD, (void(*)(int))tcp_socket_server_reaper);
  while(wait3(&status, WNOHANG, 0)>0){
    ;
  }
  
  return(0);
}
